/**
 * 3D model of double pendulum
 * author: Slavomir Tuleja, July 2003
 */

package org.opensourcephysics.tuleja.doublependulum3d;

import java.awt.*;
import java.awt.event.*;
import javax.swing.JFrame;
import org.opensourcephysics.display.*;
import org.opensourcephysics.display3d.*;
import org.opensourcephysics.numerics.*;
import org.opensourcephysics.controls.*;
import org.opensourcephysics.datapresentation.*;
import org.opensourcephysics.davidson.applets.*;


public class DoublePendulum3dApp implements ActionListener, Animation, Embeddable {
   DrawingPanel3D panel3D = new DrawingPanel3D();
   PlottingPanel panel1 = new PlottingPanel("time/s","phi1/rad","phi1 vs. time");
   PlottingPanel panel2 = new PlottingPanel("time/s","phi2/rad","phi2 vs. time");
   DataFrame frame = new DataFrame(null, null, null, null);
   AnimationControlPanel myControl = new AnimationControlPanel(this, frame);
   ObjectManager objectManager = new ObjectManager();

   DShapeGroup upperPart, stationaryPart, lowerPart;
   DBox pedestala,pedestalb,bar1a,bar1b,bar2,base;
   Dataset3D dataset;
   Dataset dataset1 = new Stripchart();
   Dataset dataset2 = new Stripchart();

   DCylinder axlea,axleb,axlec;
   DoublePendulumODE ode = new DoublePendulum();
   ODEAdaptiveSolver odeAdaptiveSolver = new RK45MultiStep(ode);
   javax.swing.Timer timer = new javax.swing.Timer(100,this);
   double L1display,L2display,tbase,wbase,hpedestal,wpedestal,d,gap,r;
   double phi1,phi2,w1,w2,t,dt,L1,L2;
   boolean trace;

   public DoublePendulum3dApp() {
      dataset1.setMarkerShape(Dataset.NO_MARKER);
      dataset1.setConnected(true);
      panel1.setPreferredMinMaxX(0,10);
      panel1.setAutoscaleX(true);
      panel1.setAutoscaleY(true);


      dataset2.setMarkerShape(Dataset.NO_MARKER);
      dataset2.setConnected(true);
      panel2.setPreferredMinMaxX(0,10);
      panel2.setAutoscaleX(true);
      panel2.setAutoscaleY(true);



      //add tabbed panels to the frame
      frame.addTabbedPanel("Virtual universe",panel3D);
      frame.addTabbedPanel("Plot phi1 vs. time",panel1);
      frame.addTabbedPanel("Plot phi2 vs. time",panel2);
      frame.addTabbedPanel("Parameters",myControl);
      myControl.setClosable(false);

      //we want the panel3D to be in front after starting the app
      frame.setVisibleComponent(panel3D);

      objectManager.clearViews();
      objectManager.addView("presentationFrame",frame);

      resetAnimation();
      initializeAnimation();

      org.opensourcephysics.display.GUIUtils.showDrawingAndTableFrames();
   }

   public void create3DScene(){
      panel3D.clear();

      //Object of the virtual universe are set up here...
      d = 0.05; // thickness of each bar
      gap = 0.05; // distance between two box parts parts connected by an axel
      r = 0.01; // radius of axels
      L1display = L1+3*r; // show upper assembly a bit longer than physical, to overlap axle
      L2display = L2+3*r; // show lower bar a bit longer than physical, to overlap axle
      hpedestal = 1.3*(L1+L2); // height of pedestal
      wpedestal = 0.1; // width of pedestal
      tbase = 0.05; // thickness of base
      wbase = 1; // width of base

      Color pedestalColor = new Color(0.4f,0.4f,0.5f);

      base = new DBox(0,0,0, wbase, tbase, wbase, pedestalColor);
      pedestala = new DBox(0,hpedestal/2.+tbase/2.,-d-2*gap-wpedestal/2., wpedestal, hpedestal, wpedestal, pedestalColor);
      axlea = new DCylinder(0, tbase/2.+0.9*hpedestal,-0.75*d-1.25*gap, r, 1.5*gap+d/2., Color.yellow);
      axlea.rotate(Math.PI/2.,0,0);
      pedestalb = new DBox(0,hpedestal/2.+tbase/2.,d+2*gap+wpedestal/2., wpedestal, hpedestal, wpedestal, pedestalColor);
      axleb = new DCylinder(0, tbase/2.+0.9*hpedestal,0.75*d+1.25*gap, r, 1.5*gap+d/2., Color.yellow);
      axleb.rotate(Math.PI/2.,0,0);

      stationaryPart = new DShapeGroup();
      stationaryPart.addShape(base);
      stationaryPart.addShape(pedestala);
      stationaryPart.addShape(axlea);
      stationaryPart.addShape(pedestalb);
      stationaryPart.addShape(axleb);

      bar1a = new DBox(0,-L1/2.,-0.75*d-gap,d,L1display,d/2.,Color.red);
      bar1b = new DBox(0,-L1/2.,0.75*d+gap,d,L1display,d/2.,Color.red);
      axlec = new DCylinder(0, -L1,0, r, 3*gap+2*d, Color.yellow);
      axlec.rotate(Math.PI/2.,0,0);

      upperPart = new DShapeGroup();
      upperPart.addShape(bar1a);
      upperPart.addShape(bar1b);
      upperPart.addShape(axlec);
      upperPart.setXYZ(0,tbase/2.+0.9*hpedestal,0);

      bar2 = new DBox(0,-L2/2.,0,d,L2display,d,Color.red);
      lowerPart = new DShapeGroup();
      lowerPart.addShape(bar2);
      lowerPart.setXYZ(0,tbase/2.+0.9*hpedestal-L1,0);

      if(trace){
         dataset = new Dataset3D();
         dataset.setConnected(true);
         dataset.setLineColor(new Color(0.9f,0.5f,0.5f,0.5f));
         dataset.setMarkerShape(Dataset3D.NO_MARKER);
         dataset.clear();
      }

      panel3D.addDrawable3D(stationaryPart);
      panel3D.addDrawable3D(upperPart);
      panel3D.addDrawable3D(lowerPart);
      if(trace) panel3D.addDrawable3D(dataset);

      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      panel3D.setSceneXYZ(-1.3715e-2,-1.3755,-4.6819);
      panel3D.setScenePEuler(-1.3163,0.0580,2.3363);

      upperPart.setPEuler(0,ode.getPhi1(),0);
      lowerPart.setXYZ(L1*Math.sin(ode.getPhi1()),tbase/2.+0.9*hpedestal-L1*Math.cos(ode.getPhi1()),0);
      lowerPart.setPEuler(0,ode.getPhi2(),0);
      if(trace) dataset.append(L1*Math.sin(ode.getPhi1())+L2*Math.sin(ode.getPhi2()),tbase/2.+0.9*hpedestal-L1*Math.cos(ode.getPhi1())-L2*Math.cos(ode.getPhi2()),0);

      panel3D.repaint();
   }

   public void actionPerformed(ActionEvent evt){
      stepAnimation();
   }

   public void initializeAnimation(){
      //method of interface Animation
      //initial conditions
      phi1=Math.toRadians(myControl.getDouble("phi1"));
      phi2=Math.toRadians(myControl.getDouble("phi2"));
      w1=Math.toRadians(myControl.getDouble("w1"));
      w2=Math.toRadians(myControl.getDouble("w2"));
      L1=myControl.getDouble("L1");
      L2=myControl.getDouble("L2");
      t=myControl.getDouble("t");
      dt=myControl.getDouble("dt");
      trace=myControl.getBoolean("show_trace");

      //initialization of ODESolver
      odeAdaptiveSolver.initialize(dt);
      odeAdaptiveSolver.setTolerance(1e-9);
      odeAdaptiveSolver.setStepSize(dt);

      ode.setPhi1(phi1); ode.setPhi2(phi2); ode.setW1(w1); ode.setW2(w2);
      ode.setL1(L1); ode.setL2(L2); ode.setT(t);

      create3DScene();

      panel3D.repaint();

      dataset1.clear();
      dataset2.clear();
      panel1.clear();
      panel2.clear();
      panel1.addDrawable(dataset1);
      panel2.addDrawable(dataset2);
      panel1.repaint();
      panel2.repaint();
   }

   public void stopAnimation(){
      //method of interface Animation
      timer.stop();
   }

   public void stepAnimation(){
      //method of interface Animation
      //execute one step of simulation
      odeAdaptiveSolver.step();

      //set the state of the pendulum
      upperPart.setPEuler(0,ode.getPhi1(),0);
      lowerPart.setXYZ(L1*Math.sin(ode.getPhi1()),tbase/2.+0.9*hpedestal-L1*Math.cos(ode.getPhi1()),0);
      lowerPart.setPEuler(0,ode.getPhi2(),0);
      if(trace) dataset.append(L1*Math.sin(ode.getPhi1())+L2*Math.sin(ode.getPhi2()),tbase/2.+0.9*hpedestal-L1*Math.cos(ode.getPhi1())-L2*Math.cos(ode.getPhi2()),0);

      dataset1.append(ode.getT(),ode.getPhi1());
      dataset2.append(ode.getT(),ode.getPhi2());

      panel3D.repaint();
      panel1.repaint();
      panel2.repaint();
   }

   public void startAnimation(){
      //method of interface Animation
      timer.start();
   }

   public void resetAnimation(){
      //method of interface Animation
      myControl.clearValues();
      myControl.setValue("phi1", 90.0);
      myControl.setValue("phi2", 90.0);
      myControl.setValue("w1", 0.0);
      myControl.setValue("w2", 0.0);
      myControl.setValue("L1", 0.5);
      myControl.setValue("L2", 1.0);
      myControl.setValue("t", 0.0);
      myControl.setValue("dt", 0.025);
      myControl.setValue("show_trace", true);
      dataset1.clear();
      dataset2.clear();
      panel1.clear();
      panel2.clear();
      panel1.addDrawable(dataset1);
      panel2.addDrawable(dataset2);
      panel1.repaint();
      panel2.repaint();
   }

   public void setControl (Control control) {
      //method of interface Animation
      myControl=(AnimationControlPanel)control;

      resetAnimation();
      initializeAnimation();
   }

   public Control getControl(){
      return myControl;
   }

   public ObjectManager getManager(){
      //metod of interface Embeddable
      return objectManager;
   }

   public static void main(String[] args) {
      Animation app = new DoublePendulum3dApp();
   }
}

class Stripchart extends Dataset{
  double range=25;

  public Stripchart(){
     super();
  }

  public double getXMin(){
     if(xmax<range) return xmin;
     else return xmax-range;
  }
}
