package org.opensourcephysics.tuleja.energydisplay;
import org.opensourcephysics.numerics.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;

/**
 * <p>Title: Energy display</p>
 * <p>Description: One dimensional motion of a particle with energy E in a potential</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Slavomir Tuleja
 */

public class PEContinuous implements Potential{
  double[] a, c, D1, D2;
  double dLeft, dRight;
  Object hotPointList;
  int n;


  public PEContinuous(double[] x, double[] y){
    int n;
    if(x.length < y.length){
      n = x.length;
    }
    else n=y.length;
    this.n=n;
    a = new double[n-1];
    c = new double[n-1];
    D1 = new double[n-1];
    D2 = new double[n-1];

    hotPointList = new LinkedList();
    for(int i = 0; i<n; i++){
      ( (java.util.List) hotPointList).add(new HotPoint(x[i], y[i]));
    }

    setCoefficients();
  }

  private void setCoefficients(){
    HotPoint hp, hp1, hp2;

    for(int i = 0; i<n-1; i++){
      hp1=(HotPoint)(((java.util.List)hotPointList).get(i));
      hp2=(HotPoint)(((java.util.List)hotPointList).get(i+1));
      D1[i] = hp1.getX()*hp1.getX()/2.0 * (hp2.getX()-hp1.getX()/3.0);
      D2[i] = hp2.getX()*hp2.getX()/2.0 * (hp1.getX()-hp2.getX()/3.0);
    }

    for(int i = 0; i<n-1; i++){
      hp1=(HotPoint)(((java.util.List)hotPointList).get(i));
      hp2=(HotPoint)(((java.util.List)hotPointList).get(i+1));
      a[i] = (hp2.getY()-hp1.getY())/(D2[i]-D1[i]);
      c[i] = (D1[i]*hp2.getY() - D2[i]*hp1.getY())/(D1[i]-D2[i]);
    }

    hp1=(HotPoint)(((java.util.List)hotPointList).get(0));
    hp2=(HotPoint)(((java.util.List)hotPointList).get(n-1));
    dLeft=(5-hp1.getY())/((-2-hp1.getX())*(-2-hp1.getX()));
    dRight=(5-hp2.getY())/((2-hp2.getX())*(2-hp2.getX()));
  }

  /**
   * Method inherited from interface Function.
   * We suppose there are 3 stationary points, x1, x2, x3, in this function. This leads to the
   * division of the definition interval of the function to 4 subintervals. On the
   * frst and the last subinterval the function is quadratic, on the remaining
   * intervals it is cubic.
   *
   * @param x x value in which the function is to be evaluated.
   * @return value of function for a given x
   */
  public double evaluate (double x){
    double y=0;
    ListIterator hpit = ( (java.util.List) hotPointList).listIterator();

    int k=0;
    HotPoint hp1, hp2;
    if (hpit.hasNext()) {
      hp1 = (HotPoint) hpit.next();
      if(x<=hp1.getX()) y=dLeft*(x-hp1.getX())*(x-hp1.getX())+hp1.getY();
      while (hpit.hasNext()) {
        hp2 = (HotPoint) hpit.next();
        if((hp1.getX()<x)&&(x<=hp2.getX())){
          y=a[k]*(x*x*x/3.0 - 0.5*(hp1.getX()+hp2.getX())*x*x + hp1.getX()*hp2.getX()*x)+c[k];
        }
        k++;
        hp1 = hp2;
      }
      if(hp1.getX()<x){
        y=dRight*(x-hp1.getX())*(x-hp1.getX())+hp1.getY();
      }
    }

    return y;
  }

  /**
   * Draws this potential using graphics context g2 and Scale object sc.
   * @param g2 graphics context
   * @param sc Scale object
   * @param showHotPoints self explanatory name
   * @param editPotential true if the hotpoints are in draggable state (they are white squares)
   */
  public void draw(Graphics2D g2, Scale sc, boolean showHotPoints, boolean editPotential){
    float[] dashed = {
        2.0f, 2.0f};
    float[] solid = {
        12.0f, 0.0f};

    BasicStroke bsDashed = new BasicStroke(1.0f, BasicStroke.CAP_ROUND,
                                           BasicStroke.JOIN_MITER, 1.0f, dashed,
                                           0.0f);
    BasicStroke bsSolid = new BasicStroke(1.0f, BasicStroke.CAP_ROUND,
                                          BasicStroke.JOIN_MITER, 1.0f, solid,
                                          0.0f);
    BasicStroke bsBold = new BasicStroke(2.0f);
    BasicStroke bsDoubleBold = new BasicStroke(4.0f);

    //potential function
    g2.setStroke(bsBold);
    double x1 = -2, x2 = 2, dx = 0.025;
    double x = x1;
    while (x <= x2 - dx) {
      g2.drawLine(sc.xToi(x), sc.yToj(evaluate(x)),
                  sc.xToi(x + dx), sc.yToj(evaluate(x + dx)));
      x += dx;
    }
    g2.setStroke(bsSolid);

    //hot points on the potential function
    if (showHotPoints) {
      ListIterator hpit = ( (java.util.List) hotPointList).listIterator();

      while (hpit.hasNext()) {
        HotPoint sp = (HotPoint) hpit.next();
        int i = sc.xToi(sp.getX());
        int j = sc.yToj(sp.getY());
        g2.setColor(Color.red);
        g2.fillRect(i - 8, j - 8, 16, 16);
        g2.setColor(Color.black);
        g2.drawRect(i - 8, j - 8, 16, 16);
      }
    }
  }

  /**
   * Returns a LinkedList object of hot points, or stationary points of this function.
   * @return LinkedList object of StationaryPoint objects
   */
  public Object getHotPointList(){
    return hotPointList;
  }

  /**
   * Used to set new positions of stationary points from the outside. There are
   * three cases depending on which stationary point is currently set.
   * @param i index of stationary point just being adjusted
   * @param x new x-coordinate of the stationary point
   * @param y new y-coordinate of the stationary point
   */
  public void setHotPoint(int i, double x, double y){
    HotPoint hp, hp1, hp2;
     if(i==1){
       hp=(HotPoint)(((java.util.List)hotPointList).get(0));
       hp2=(HotPoint)(((java.util.List)hotPointList).get(1));
       if((x<hp2.getX()-0.02)&&(x>-2+0.02)) hp.x=x;
       hp.y=y;
     }
     else if(i==n){
       hp=(HotPoint)(((java.util.List)hotPointList).get(n-1));
       hp1=(HotPoint)(((java.util.List)hotPointList).get(n-2));
       if((x<2-0.02)&&(x>hp1.getX()+0.02)) hp.x=x;
       hp.y=y;
     }
     else {
       hp=(HotPoint)(((java.util.List)hotPointList).get(i-1));
       hp1=(HotPoint)(((java.util.List)hotPointList).get(i-2));
       hp2=(HotPoint)(((java.util.List)hotPointList).get(i));

       if((x<hp2.getX()-0.02)&&(x>hp1.getX()+0.02)) hp.x=x;
       hp.y=y;
     }

     setCoefficients();
  }
}


